@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010,2011,2014,2016,2018 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010,2011,2014,2016,2018,2019 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -40,19 +40,21 @@
 /**                based architecture from a source graph. **/
 /**                                                        **/
 /**   DATES      : # Version 3.2  : from : 29 may 1997     **/
-/**                                 to     30 aug 1998     **/
+/**                                 to   : 30 aug 1998     **/
 /**                # Version 3.3  : from : 01 oct 1998     **/
-/**                                 to     01 oct 1998     **/
+/**                                 to   : 01 oct 1998     **/
 /**                # Version 3.4  : from : 30 oct 2001     **/
-/**                                 to     08 nov 2001     **/
+/**                                 to   : 08 nov 2001     **/
 /**                # Version 4.0  : from : 29 nov 2003     **/
-/**                                 to     10 mar 2005     **/
+/**                                 to   : 10 mar 2005     **/
 /**                # Version 5.0  : from : 10 sep 2007     **/
-/**                                 to     03 apr 2008     **/
+/**                                 to   : 03 apr 2008     **/
 /**                # Version 5.1  : from : 28 sep 2008     **/
-/**                                 to     28 jun 2011     **/
+/**                                 to   : 28 jun 2011     **/
 /**                # Version 6.0  : from : 28 jun 2011     **/
-/**                                 to     15 may 2018     **/
+/**                                 to   : 15 may 2018     **/
+/**                # Version 7.0  : from : 21 aug 2019     **/
+/**                                 to   : 12 sep 2019     **/
 /**                                                        **/
 /************************************************************/
 
@@ -121,7 +123,8 @@ archDecoArchBuild (
 Arch * restrict const       tgtarchptr,           /*+ Decomposition architecture to build    +*/
 const Graph * const         tgtgrafptr,           /*+ Source graph modeling the architecture +*/
 const VertList * const      tgtlistptr,           /*+ Subset of source graph vertices        +*/
-const Strat * const         mapstrat)             /*+ Bipartitioning strategy                +*/
+const Strat * const         mapstrat,             /*+ Bipartitioning strategy                +*/
+Context * const             contptr)              /*+ Execution context                      +*/
 {
   Arch                              archdat;      /* Variable-sized architecture for bipartitioning */
   ArchDom                           domsub0;      /* Temporary space for subdomain 0                */
@@ -159,8 +162,6 @@ const Strat * const         mapstrat)             /*+ Bipartitioning strategy
   if (termdomnbr == 0)                            /* If nothing to do */
     return (0);
 
-  intRandInit ();                                 /* Initialize random generator */
-
   invedlosiz = (tgtedlotax != NULL) ? tgtgrafptr->edgenbr : 0;
   if ((memAllocGroup ((void **) (void *)
                       &jobtab,     (size_t) (termdomnbr * sizeof (ArchDecoBuildJob)),
@@ -208,6 +209,7 @@ const Strat * const         mapstrat)             /*+ Bipartitioning strategy
   actgrafdat.veextax = NULL;                      /* No external gain array      */
   actgrafdat.parttax = actparttax;                /* Set global auxiliary arrays */
   actgrafdat.frontab = actfrontab;
+  actgrafdat.contptr = contptr;                   /* Use same context for all jobs  */
   joblink = NULL;                                 /* Initialize job list            */
   if (jobtab[0].grafdat.vertnbr > 1) {            /* If job is worth bipartitioning */
     jobtab[0].joblink = joblink;                  /* Add initial job to list        */
