@@ -1,4 +1,4 @@
-/* Copyright 2007-2009,2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2007-2009,2012,2014,2018,2020,2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -45,6 +45,10 @@
 /**                                 to   : 26 may 2009     **/
 /**                # Version 6.0  : from : 18 sep 2012     **/
 /**                                 to   : 07 jun 2018     **/
+/**                # Version 6.1  : from : 17 jun 2021     **/
+/**                                 to   : 17 jun 2021     **/
+/**                # Version 7.0  : from : 14 jan 2020     **/
+/**                                 to   : 29 dec 2021     **/
 /**                                                        **/
 /************************************************************/
 
@@ -85,9 +89,9 @@ typedef struct DgraphCoarsenVert_ {
    the amount of data to be sent to each neighbor process. */
 
 typedef struct DgraphCoarsenCount_ {
-  Gnum                      vertsndnbr;
-  Gnum                      edgesndnbr;
-  Gnum                      vertlocnbr;
+  Gnum                      vertsndnbr;           /*+ Number of vertices to send to this neighbor +*/
+  Gnum                      edgesndnbr;           /*+ Number of edges to send to this neighbor    +*/
+  Gnum                      vertlocnbr;           /*+ Number of local vertices in this neighbor   +*/
 } DgraphCoarsenCount;
 
 /*+ A table made of such elements is used during
@@ -101,6 +105,21 @@ typedef struct DgraphCoarsenHash_ {
 } DgraphCoarsenHash;
 
 
+/*+ The thread-specific data block, which
+    has two uses. Firstly, it helps define
+    the repartition across threads of the
+    adjacency lists received from the
+    different neighbors. Secondly, it
+    allows reduction of graph properties.  +*/
+
+typedef struct DgraphCoarsenThread_  {
+  Gnum                      edgelocsum;           /*+ Upper bound on sum of local (fine) edges for this thread and below  +*/
+  Gnum                      velolocsum;           /*+ Sum of vertex loads for each thread, for final reduce-sum operation +*/
+  Gnum                      edgelocnbr;           /*+ Number of edges for each thread, for final reduce-sum operation     +*/
+  Gnum                      degrlocmax;           /*+ Maximum degree for each thread, for final reduce-max operation      +*/
+  int                       retuval;              /*+ Return value                                                        +*/
+} DgraphCoarsenThread;
+
 /*+ This structure gathers all data necessary
     to the proper execution of the coarsening
     and matching routines.                    +*/
@@ -128,11 +147,23 @@ typedef struct DgraphCoarsenData_ {
   DgraphCoarsenMulti *      multloctmp;           /*+ Pointer to multloctab structure to free (if any)             +*/
   DgraphCoarsenMulti *      multloctab;           /*+ Structure which contains the result of the matching          +*/
   Gnum                      multlocnbr;           /*+ Index of next multinode to be created                        +*/
+#ifdef SCOTCH_DEBUG_DGRAPH2
+  Gnum                      multlocsiz;           /*+ Size of allocated multinode array                            +*/
+#endif /* SCOTCH_DEBUG_DGRAPH2 */
   Gnum                      vertrcvnbr;           /*+ Number of fine vertices to be received                       +*/
   Gnum                      edgercvnbr;           /*+ Number of fine edges to be received                          +*/
   Gnum                      edgekptnbr;           /*+ Upper bound on number of edges kept from finer graph         +*/
   Gnum                      vertsndnbr;           /*+ Number of fine vertices to be sent                           +*/
   Gnum                      edgesndnbr;           /*+ Number of fine edges to be sent                              +*/
+  Gnum *                    ercvdattab;           /*+ Pointer to adjacency exchange receive array                  +*/
+  int *                     ercvdsptab;           /*+ Displacement data for adjacency exchange receive array       +*/
+#if (defined SCOTCH_PTHREAD) && (! defined DGRAPHCOARSENNOTHREAD)
+  int *                     ercvcnttab;           /*+ Count data for adjacency exchange receive array              +*/
+#endif /* (defined SCOTCH_PTHREAD) && (! defined DGRAPHCOARSENNOTHREAD) */
+  Gnum                      degrlocmax;           /*+ Local maximum degree of coarse graph                         +*/
+  Gnum                      coarhashmsk;          /*+ Mask value of adjacency hash table (TRICK: size - 1)         +*/
+  DgraphCoarsenThread *     thrdtab;              /*+ Optional array for adjacency splitting across threads        +*/
+  Context *                 contptr;              /*+ Execution context                                            +*/
 } DgraphCoarsenData;
 
 /*
@@ -145,4 +176,4 @@ static void                 dgraphCoarsenExit   (DgraphCoarsenData * restrict co
 static int                  dgraphCoarsenBuild  (DgraphCoarsenData * restrict const);
 #endif /* DGRAPH_COARSEN */
 
-int                         dgraphCoarsen       (Dgraph * restrict const, Dgraph * restrict const, DgraphCoarsenMulti * restrict * const, const Gnum, const Gnum, const double, const int);
+int                         dgraphCoarsen       (Dgraph * restrict const, Dgraph * restrict const, DgraphCoarsenMulti * restrict * const, const Gnum, const Gnum, const double, const int, Context * restrict const);
