@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010,2012,2014,2018,2020 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010,2012,2014,2018-2020 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -39,13 +39,15 @@
 /**                graph subgraph-making functions.        **/
 /**                                                        **/
 /**   DATES      : # Version 4.0  : from : 02 jan 2002     **/
-/**                                 to     25 feb 2004     **/
+/**                                 to   : 25 feb 2004     **/
 /**                # Version 5.0  : from : 19 dec 2006     **/
-/**                                 to     11 jun 2008     **/
+/**                                 to   : 11 jun 2008     **/
 /**                # Version 5.1  : from : 24 oct 2010     **/
-/**                                 to     24 oct 2010     **/
+/**                                 to   : 24 oct 2010     **/
 /**                # Version 6.0  : from : 27 mar 2012     **/
-/**                                 to     20 aug 2020     **/
+/**                                 to   : 20 aug 2020     **/
+/**                # Version 7.0  : from : 05 may 2019     **/
+/**                                 to   : 05 may 2019     **/
 /**                                                        **/
 /************************************************************/
 
@@ -208,6 +210,7 @@ Gnum * restrict const           indedgetab)       /* Pointer to pre-allocated sp
   }
   indgrafptr->s.vendtax = indgrafptr->s.verttax + 1; /* Use compact representation of arrays */
   indgrafptr->levlnum   = orggrafptr->levlnum + 1; /* Induced subgraph is one level below    */
+  indgrafptr->contptr   = orggrafptr->contptr;    /* Use same execution context              */
 
   if (orggrafptr->s.vnumtax != NULL) {            /* Adjust vnumtax */
     const Gnum * restrict const orgvnumtax = orggrafptr->s.vnumtax;
