@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2018 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007,2008,2018-2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -46,6 +46,8 @@
 /**                                 to   : 09 nov 2008     **/
 /**                # Version 6.0  : from : 15 may 2018     **/
 /**                                 to   : 15 may 2018     **/
+/**                # Version 7.0  : from : 12 sep 2019     **/
+/**                                 to   : 30 may 2021     **/
 /**                                                        **/
 /************************************************************/
 
@@ -67,6 +69,48 @@
 #include "vmesh.h"
 #include "vmesh_separate_st.h"
 
+/***************************/
+/*                         */
+/* Multi-threaded routine. */
+/*                         */
+/***************************/
+
+static
+void
+hmeshOrderNd2 (
+Context * restrict const        contptr,          /*+ (Sub-)context                          +*/
+const int                       spltnum,          /*+ Rank of sub-context in initial context +*/
+const HmeshOrderNdSplit * const spltptr)
+{
+  Hmesh               orgmeshdat;
+  Hmesh               indmeshdat;
+  int                 o;
+
+  orgmeshdat = *spltptr->meshptr;                 /* Assign new context to work mesh */
+  orgmeshdat.contptr = contptr;
+
+  o = 0;
+  if ((hmeshInducePart (&orgmeshdat, spltptr->parttax, spltnum, spltptr->splttab[spltnum].velmnbr,
+                        spltptr->splttab[spltnum].vnodnbr, spltptr->vnspnbr, &indmeshdat)) != 0) {
+    errorPrint ("hmeshOrderNd2: cannot build induced submesh");
+    o = 1;
+  }
+  if (o == 0) {
+    o = hmeshOrderNd (&indmeshdat, spltptr->ordeptr, spltptr->splttab[spltnum].ordenum, spltptr->splttab[spltnum].cblkptr, spltptr->paraptr);
+
+    hmeshExit (&indmeshdat);
+  }
+  if (o != 0) {                                   /* Report any error */
+#ifdef SCOTCH_PTHREAD
+    pthread_mutex_lock (&spltptr->ordeptr->mutedat); /* Use ordering lock to avoid race condition */
+#endif /* SCOTCH_PTHREAD */
+    *spltptr->revaptr = 1;
+#ifdef SCOTCH_PTHREAD
+    pthread_mutex_unlock (&spltptr->ordeptr->mutedat);
+#endif /* SCOTCH_PTHREAD */
+  }
+}
+
 /*****************************/
 /*                           */
 /* This is the main routine. */
@@ -87,8 +131,7 @@ const Gnum                        ordenum,
 OrderCblk * restrict const        cblkptr,
 const HmeshOrderNdParam * const   paraptr)
 {
-  Hmesh                     indmeshdat;           /* Induced halo mesh data */
-  Vmesh                     nspmeshdat;           /* Node separation mesh   */
+  Vmesh                     nspmeshdat;           /* Node separation mesh */
   Gnum                      vertnbr;
   int                       o;
 
@@ -106,6 +149,7 @@ const HmeshOrderNdParam * const   paraptr)
   nspmeshdat.ncmpsize[1] = 0;
   nspmeshdat.fronnbr     = 0;
   nspmeshdat.levlnum     = meshptr->levlnum;
+  nspmeshdat.contptr     = meshptr->contptr;
 
   vertnbr = nspmeshdat.m.velmnbr + nspmeshdat.m.vnodnbr;
   if (memAllocGroup ((void **) (void *)
@@ -147,10 +191,18 @@ const HmeshOrderNdParam * const   paraptr)
   cblkptr->cblktab[2].cblknbr = 0;
   cblkptr->cblktab[2].cblktab = NULL;
 
-  if (nspmeshdat.fronnbr != 0) {                  /* If separator not empty         */
-    cblkptr->cblknbr  = 3;                        /* It is a three-cell tree node   */
+  if (nspmeshdat.fronnbr != 0) {                  /* If separator not empty */
+    Hmesh               indmeshdat;
+
+    cblkptr->cblknbr = 3;                         /* It is a three-cell tree node */
+#ifdef SCOTCH_PTHREAD
+    pthread_mutex_lock (&ordeptr->mutedat);
+#endif /* SCOTCH_PTHREAD */
     ordeptr->cblknbr += 2;                        /* Two more column blocks created */
     ordeptr->treenbr += 3;                        /* Three more tree nodes created  */
+#ifdef SCOTCH_PTHREAD
+    pthread_mutex_unlock (&ordeptr->mutedat);
+#endif /* SCOTCH_PTHREAD */
 
     cblkptr->cblktab[2].typeval = ORDERCBLKOTHR;
     cblkptr->cblktab[2].vnodnbr = nspmeshdat.fronnbr;
@@ -168,36 +220,50 @@ const HmeshOrderNdParam * const   paraptr)
     indmeshdat.vnhlsum = indmeshdat.m.vnlosum;
     indmeshdat.enohnbr = indmeshdat.m.edgenbr;
     indmeshdat.levlnum = meshptr->levlnum;        /* Separator mesh is at level of original mesh */
+    indmeshdat.contptr = meshptr->contptr;
 
     o = hmeshOrderSt (&indmeshdat, ordeptr, ordenum + nspmeshdat.ncmpsize[0] + nspmeshdat.ncmpsize[1],
                       cblkptr->cblktab + 2, paraptr->ordstratsep);
     hmeshExit (&indmeshdat);
   }
-  else {                                          /* Separator is empty             */
-    cblkptr->cblknbr = 2;                         /* It is a two-cell tree node     */
-    ordeptr->cblknbr ++;                          /* One more column block created  */
-    ordeptr->treenbr += 2;                        /* Two more tree nodes created    */
+  else {                                          /* Separator is empty         */
+    cblkptr->cblknbr = 2;                         /* It is a two-cell tree node */
+#ifdef SCOTCH_PTHREAD
+    pthread_mutex_lock (&ordeptr->mutedat);
+#endif /* SCOTCH_PTHREAD */
+    ordeptr->cblknbr ++;                          /* One more column block created */
+    ordeptr->treenbr += 2;                        /* Two more tree nodes created   */
+#ifdef SCOTCH_PTHREAD
+    pthread_mutex_unlock (&ordeptr->mutedat);
+#endif /* SCOTCH_PTHREAD */
     o = 0;                                        /* No separator ordering computed */
   }
   if (o == 0) {
-    if (hmeshInducePart (meshptr, nspmeshdat.parttax, 0, nspmeshdat.ecmpsize[0],
-                         nspmeshdat.ncmpsize[0], nspmeshdat.fronnbr, &indmeshdat) != 0) {
-      errorPrint ("hmeshOrderNd: cannot build induced subgraph (2)");
-      memFree    (nspmeshdat.frontab);            /* Free remaining space */
-      return     (1);
+    HmeshOrderNdSplit   spltdat;                  /* Parameters for context splitting */
+
+    spltdat.splttab[0].velmnbr = nspmeshdat.ecmpsize[0];
+    spltdat.splttab[0].vnodnbr = nspmeshdat.ncmpsize[0];
+    spltdat.splttab[0].ordenum = ordenum;
+    spltdat.splttab[0].cblkptr = cblkptr->cblktab;
+    spltdat.splttab[1].velmnbr = nspmeshdat.ecmpsize[1];
+    spltdat.splttab[1].vnodnbr = nspmeshdat.ncmpsize[1];
+    spltdat.splttab[1].ordenum = ordenum + nspmeshdat.ncmpsize[0];
+    spltdat.splttab[1].cblkptr = cblkptr->cblktab + 1;
+    spltdat.meshptr = meshptr;
+    spltdat.vnspnbr = nspmeshdat.fronnbr;
+    spltdat.parttax = nspmeshdat.parttax;
+    spltdat.ordeptr = ordeptr;
+    spltdat.paraptr = paraptr;
+    spltdat.revaptr = &o;
+
+#ifndef HMESHORDERNDNOTHREAD
+    if (contextThreadLaunchSplit (meshptr->contptr, (ContextSplitFunc) hmeshOrderNd2, &spltdat) != 0) /* If counld not split context to run concurrently */
+#endif /* HMESHORDERNDNOTHREAD */
+    {
+      hmeshOrderNd2 (meshptr->contptr, 0, &spltdat); /* Run tasks in sequence */
+      if (o == 0)
+        hmeshOrderNd2 (meshptr->contptr, 1, &spltdat);
     }
-    o = hmeshOrderNd (&indmeshdat, ordeptr, ordenum, cblkptr->cblktab, paraptr);
-    hmeshExit (&indmeshdat);
-  }
-  if (o == 0) {
-    if (hmeshInducePart (meshptr, nspmeshdat.parttax, 1, nspmeshdat.ecmpsize[1],
-                         nspmeshdat.ncmpsize[1], nspmeshdat.fronnbr, &indmeshdat) != 0) {
-      errorPrint ("hmeshOrderNd: cannot build induced subgraph (3)");
-      memFree    (nspmeshdat.frontab);            /* Free remaining space */
-      return     (1);
-    }
-    o = hmeshOrderNd (&indmeshdat, ordeptr, ordenum + nspmeshdat.ncmpsize[0], cblkptr->cblktab + 1, paraptr);
-    hmeshExit (&indmeshdat);
   }
 
   vmeshExit (&nspmeshdat);
