@@ -1,4 +1,4 @@
-/* Copyright 2010,2011,2013,2014 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2010,2011,2013,2014,2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -40,9 +40,11 @@
 /**                consistency checking routine.           **/
 /**                                                        **/
 /**   DATES      : # Version 5.1  : from : 13 jul 2010     **/
-/**                                 to     13 jul 2010     **/
+/**                                 to   : 13 jul 2010     **/
 /**                # Version 6.0  : from : 03 mar 2011     **/
-/**                                 to     14 sep 2014     **/
+/**                                 to   : 14 sep 2014     **/
+/**                # Version 7.0  : from : 11 jul 2021     **/
+/**                                 to   : 11 jul 2021     **/
 /**                                                        **/
 /************************************************************/
 
@@ -102,14 +104,14 @@ const Kgraph * restrict const grafptr)
 
   if (&grafptr->s != grafptr->m.grafptr) {
     errorPrint ("kgraphCheck: invalid mapping graph");
-    return     (1);
+    return (1);
   }
 
   if ((grafptr->m.domnmax <= 0)                 ||
       (grafptr->m.domnnbr > grafptr->m.domnmax) ||
       (grafptr->m.domnnbr > grafptr->s.vertnbr)) {
     errorPrint ("kgraphCheck: invalid number of domains");
-    return     (1);
+    return (1);
   }
 
   if (memAllocGroup ((void **) (void *)
@@ -198,7 +200,7 @@ const Kgraph * restrict const grafptr)
     Gnum                commcut;
 
     partval = parttax[vertnum];
-    tdomnum = archDomNum (&grafptr->a, &grafptr->m.domntab[partval]);
+    tdomnum = archDomNum (grafptr->m.archptr, &grafptr->m.domntab[partval]);
     tdfinum = (grafptr->pfixtax != NULL) ? grafptr->pfixtax[vertnum] : -1;
 
     if ((tdfinum != -1) &&                        /* If it is a fixed vertex */
