@@ -1,4 +1,4 @@
-/* Copyright 2011,2013,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2011,2013,2014,2018,2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -41,6 +41,8 @@
 /**                                                        **/
 /**   DATES      : # Version 6.0  : from : 27 may 2011     **/
 /**                                 to   : 06 jun 2018     **/
+/**                # Version 7.0  : from : 11 jul 2021     **/
+/**                                 to   : 11 jul 2021     **/
 /**                                                        **/
 /************************************************************/
 
@@ -83,9 +85,7 @@ const KgraphMapExParam * const  paraptr)          /*+ Method parameters +*/
   Anum * restrict             parttax;
   Anum                        treemax;            /* Maximum number of nodes in tree structure */
   Anum                        treenbr;            /* Number of nodes in tree structure         */
-  const Arch * restrict       archptr;
   ArchDom                     domndat;            /* Root domain                               */
-  Anum                        domnnbr;
   Anum                        domnnum;
   Gnum                        sortnbr;            /* Number of non-fixed vertices              */
   Gnum                        sortnum;
@@ -98,14 +98,14 @@ const KgraphMapExParam * const  paraptr)          /*+ Method parameters +*/
   int                         flagval;            /* Flag unset if load imbalance to fix       */
 
   const Gnum * restrict const velotax = grafptr->s.velotax;
+  const Arch * restrict const archptr = grafptr->m.archptr;
+  const Anum                  domnnbr = grafptr->m.domnnbr;
   const Anum * restrict const pfixtax = grafptr->pfixtax;
 
-  archptr = grafptr->m.archptr;
   archDomFrst (archptr, &domndat);
 
   grafptr->kbalval = paraptr->kbalval;            /* Store last k-way imbalance ratio */
-  domnnbr = grafptr->m.domnnbr;
-  sortnbr = grafptr->s.vertnbr - grafptr->vfixnbr; /* Only sort non-fixed vertices */
+  sortnbr = grafptr->s.vertnbr - grafptr->vfixnbr; /* Only sort non-fixed vertices    */
   treemax = 2 * ((archVar (archptr)) ? domnnbr : archDomSize (archptr, &domndat)); /* At most twice the number of terminals in last rank */
   if (memAllocGroup ((void **) (void *)
                      &doextab, (size_t) (domnnbr * sizeof (KgraphMapExDom)),
@@ -113,7 +113,7 @@ const KgraphMapExParam * const  paraptr)          /*+ Method parameters +*/
                      &termtab, (size_t) (domnnbr * sizeof (KgraphMapExTerm)),
                      &treetab, (size_t) (treemax * sizeof (KgraphMapExTree)), NULL) == NULL) {
     errorPrint ("kgraphMapEx: out of memory");
-    return     (1);
+    return (1);
   }
 
   wghtsum = (double) archDomWght (archptr, &domndat);
@@ -168,7 +168,7 @@ const KgraphMapExParam * const  paraptr)          /*+ Method parameters +*/
 #ifdef SCOTCH_DEBUG_KGRAPH2
   if (treenbr >= treemax) {
     errorPrint ("kgraphMapEx: internal error (1)");
-    return     (1);
+    return (1);
   }
 #endif /* SCOTCH_DEBUG_KGRAPH2 */
 
@@ -189,7 +189,7 @@ const KgraphMapExParam * const  paraptr)          /*+ Method parameters +*/
 #ifdef SCOTCH_DEBUG_KGRAPH2
   if (sortnbr != (grafptr->s.vertnbr - grafptr->vfixnbr)) {
     errorPrint ("kgraphMapEx: internal error (2)");
-    return     (1);
+    return (1);
   }
 #endif /* SCOTCH_DEBUG_KGRAPH2 */
   if (velotax != NULL)                            /* If vertices are weighted, sort them in ascending order */
@@ -224,7 +224,7 @@ const KgraphMapExParam * const  paraptr)          /*+ Method parameters +*/
 #ifdef SCOTCH_DEBUG_KGRAPH2
   if (kgraphCheck (grafptr) != 0) {
     errorPrint ("kgraphMapEx: inconsistent graph data");
-    return     (1);
+    return (1);
   }
 #endif /* SCOTCH_DEBUG_KGRAPH2 */
 
@@ -263,7 +263,7 @@ const ArchDom * restrict const          domnptr)  /*+ Pointer to subdomain to co
 #ifdef SCOTCH_DEBUG_KGRAPH2
     if (o != 0) {
       errorPrint ("kgraphMapExTree: internal error");
-      return     (-1);
+      return (-1);
     }
 #endif /* SCOTCH_DEBUG_KGRAPH2 */
     sonstab[0] = kgraphMapExTree (archptr, termtab, termnbr, doextab, treetab, treeptr, &domntab[0]);
@@ -296,7 +296,7 @@ const ArchDom * restrict const          domnptr)  /*+ Pointer to subdomain to co
 #ifdef SCOTCH_DEBUG_KGRAPH2
     if (archVar (archptr)) {
       errorPrint ("kgraphMapExTree: not implemented");
-      return     (-1);
+      return (-1);
     }
 #endif /* SCOTCH_DEBUG_KGRAPH2 */
 
