@@ -1,4 +1,4 @@
-/* Copyright 2004,2007-2009,2011,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007-2009,2011,2014,2018,2019,2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -67,6 +67,10 @@
 /**                                 to   : 04 feb 2009     **/
 /**                # Version 6.0  : from : 03 mar 2011     **/
 /**                                 to   : 21 jun 2019     **/
+/**                # Version 6.1  : from : 28 jun 2021     **/
+/**                                 to   : 28 jun 2021     **/
+/**                # Version 7.0  : from : 25 aug 2019     **/
+/**                                 to   : 27 jul 2021     **/
 /**                                                        **/
 /**   NOTES      : # This code is a complete rewrite of    **/
 /**                  the original code of kgraphMapRb(),   **/
@@ -115,7 +119,8 @@ static
 int
 kgraphMapRbMapPoolInit (
 KgraphMapRbMapPoolData * restrict const poolptr,
-const KgraphMapRbData * restrict const  dataptr)
+const KgraphMapRbData * restrict const  dataptr,
+Context * const                         contptr)
 {
   int                 flagval;
 
@@ -144,7 +149,7 @@ const KgraphMapRbData * restrict const  dataptr)
 
   if ((poolptr->jobtab = (KgraphMapRbMapJob *) memAlloc (mappptr->domnmax * sizeof (KgraphMapRbMapJob))) == NULL) {
     errorPrint ("kgraphMapRbMapPoolInit: out of memory (2)");
-    return     (1);
+    return (1);
   }
   poolptr->jobtab[0].poolflag = 0;                /* In case kgraphMapRbPoolExit() is called just afterwards on single-domain mapping */
 
@@ -159,11 +164,12 @@ const KgraphMapRbData * restrict const  dataptr)
     if ((poolptr->domntab[1] = (ArchDom *) memAlloc (mappptr->domnmax * sizeof (ArchDom))) == NULL) {
       errorPrint ("kgraphMapRbMapPoolInit: out of memory (3)");
       memFree    (poolptr->jobtab);
-      return     (1);
+      return (1);
     }
   }
 
   poolptr->flagval = flagval;
+  poolptr->contptr = contptr;
 
   return (0);
 }
@@ -252,7 +258,7 @@ KgraphMapRbMapPoolData * restrict const poolptr)
 
   if ((jobtab = (KgraphMapRbMapJob *) memRealloc (poolptr->jobtab, domnmax * sizeof (KgraphMapRbMapJob))) == NULL) {
     errorPrint ("kgraphMapRbMapPoolResize: out of memory (1)");
-    return     (1);
+    return (1);
   }
   if (jobtab != poolptr->jobtab) {                /* If job array moved                */
     KgraphMapRbMapJob *       joboldtab;          /* Pointer to start of old job array */
@@ -285,7 +291,7 @@ KgraphMapRbMapPoolData * restrict const poolptr)
   i = (poolptr->domntab[1] == poolptr->mappptr->domntab) ? 1 : 0; /* Find which domain array is that of the mapping */
   if (mapResize (poolptr->mappptr, domnmax) != 0) {
     errorPrint ("kgraphMapRbMapPoolResize: out of memory (2)");
-    return     (1);
+    return (1);
   }
   if (poolptr->domntab[1] != poolptr->domntab[0]) { /* If two domain arrays present */
     ArchDom *           domntab;
@@ -378,7 +384,7 @@ KgraphMapRbMapJob * const       jobptr)           /* Job to be added */
   switch (poolptr->polival) {                     /* Set job priority value */
     case KGRAPHMAPRBPOLIRANDOM :
       jobptr->prioval =
-      jobptr->priolvl = intRandVal (INTVALMAX);
+      jobptr->priolvl = contextIntRandVal (poolptr->contptr, INTVALMAX);
       break;
     case KGRAPHMAPRBPOLILEVEL   :
     case KGRAPHMAPRBPOLINGLEVEL :
@@ -429,7 +435,7 @@ const GraphPart                 partval)
   switch (poolptr->polival) {                     /* Set job priority value */
     case KGRAPHMAPRBPOLIRANDOM :
       prioval =
-      priolvl = intRandVal (INTVALMAX);
+      priolvl = contextIntRandVal (poolptr->contptr, INTVALMAX);
       break;
     case KGRAPHMAPRBPOLILEVEL :
       priolvl = joboldptr->priolvl + 1;
@@ -441,13 +447,11 @@ const GraphPart                 partval)
     case KGRAPHMAPRBPOLINGSIZE :
       prioval = jobnewptr->grafdat.vertnbr;
       break;
-#ifdef SCOTCH_DEBUG_KGRAPH2
     default :
       errorPrint ("kgraphMapRbMapPoolUpdt1: unknown job selection policy");
       jobnewptr->prioval = 0;
       jobnewptr->priolvl = 0;
       return;
-#endif /* SCOTCH_DEBUG_KGRAPH2 */
   }
 
   jobnewptr->prioval = prioval;
@@ -595,7 +599,7 @@ KgraphMapRbMapJob * const       jobnewptr1)
     switch (poolptr->polival) {                   /* Set job priority value */
       case KGRAPHMAPRBPOLIRANDOM :
         prioval =
-        priolvl = intRandVal (INTVALMAX);
+        priolvl = contextIntRandVal (poolptr->contptr, INTVALMAX);
         break;
       case KGRAPHMAPRBPOLILEVEL :
         priolvl = joboldptr->priolvl + 1;
@@ -607,13 +611,11 @@ KgraphMapRbMapJob * const       jobnewptr1)
       case KGRAPHMAPRBPOLINGSIZE :
         prioval = jobnewptr->grafdat.vertnbr;
         break;
-#ifdef SCOTCH_DEBUG_KGRAPH2
       default :
         errorPrint ("kgraphMapRbMapPoolUpdt2: unknown job selection policy");
         jobnewptr->prioval = 0;
         jobnewptr->priolvl = 0;
         return;
-#endif /* SCOTCH_DEBUG_KGRAPH2 */
     }
 
     jobnewptr0->prioval = prioval + 1;            /* TRICK: when processing subdomain 1, subdomain 0 has higher priority value */
@@ -862,7 +864,8 @@ kgraphMapRbMap (
 const KgraphMapRbData * restrict const  dataptr,  /*+ Global mapping data                  +*/
 const Graph * restrict const            grafptr,  /*+ Graph to map, without fixed vertices +*/
 const Anum                              vflonbr,  /*+ Number of fixed vertex load slots    +*/
-KgraphMapRbVflo * restrict const        vflotab)  /*+ Array of fixed vertex load slots     +*/
+KgraphMapRbVflo * restrict const        vflotab,  /*+ Array of fixed vertex load slots     +*/
+Context * const                         contptr)  /*+ Execution context                    +*/
 {
   KgraphMapRbMapPoolData  pooldat;                /* Data for handling jobs and job pools */
   ArchDom                 domnsubtab[2];          /* Subdomains of current job domain     */
@@ -893,7 +896,7 @@ KgraphMapRbVflo * restrict const        vflotab)  /*+ Array of fixed vertex load
   mappptr->domnmax = 1;                           /* Force resizing of job arrays, for debugging */
 #endif /* SCOTCH_DEBUG_KGRAPH2 */
 
-  if (kgraphMapRbMapPoolInit (&pooldat, dataptr) != 0) /* Initialize pool data; done first for kgraphMapRbMapPoolExit() to succeed afterwards */
+  if (kgraphMapRbMapPoolInit (&pooldat, dataptr, contptr) != 0) /* Initialize pool data; done first for kgraphMapRbMapPoolExit() to succeed afterwards */
     return (1);
 
   if ((((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) == 0) && (archDomSize (mappptr->archptr, &mappptr->domnorg) <= 1)) || /* If single-vertex domain   */
@@ -907,7 +910,9 @@ KgraphMapRbVflo * restrict const        vflotab)  /*+ Array of fixed vertex load
   pooldat.jobtab[0].grafdat.flagval &= ~GRAPHFREETABS; /* Do not free its arrays on exit     */
   pooldat.jobtab[0].vflonbr = vflonbr;            /* Record initial list of fixed load slots */
   pooldat.jobtab[0].vflotab = vflotab;
-  kgraphMapRbMapPoolFrst (&pooldat, &pooldat.jobtab[0]); /* Add initial job */
+  pooldat.jobtab[0].vflotab = vflotab;
+  pooldat.jobtab[0].levlnum = 0;                  /* Initial recursion level is 0 */
+  kgraphMapRbMapPoolFrst (&pooldat, &pooldat.jobtab[0]); /* Add initial job       */
 
   comploadmin = (1.0 - dataptr->paraptr->kbalval) * dataptr->comploadrat; /* Ratio can have been tilted when working on subgraph */
   comploadmax = (1.0 + dataptr->paraptr->kbalval) * dataptr->comploadrat;
@@ -918,6 +923,7 @@ KgraphMapRbVflo * restrict const        vflotab)  /*+ Array of fixed vertex load
     while ((joborgptr = kgraphMapRbMapPoolGet (&pooldat)) != NULL) { /* For all jobs in pool */
       Gnum                vflonbrtab[2];
       Gnum                vflowgttab[2];
+      Gnum                levlnum;
       int                 partval;
 
       jobsubnum[0] = joborgptr - pooldat.jobtab;  /* Get current (and first son) job slot number before possible move of pointers */
@@ -931,7 +937,7 @@ KgraphMapRbVflo * restrict const        vflotab)  /*+ Array of fixed vertex load
 
       kgraphMapRbVfloSplit (mappptr->archptr, domnsubtab, /* Split fixed vertex load slots, if any */
                             joborgdat.vflonbr, joborgdat.vflotab, vflonbrtab, vflowgttab);
-      if (kgraphMapRbBgraph (dataptr, &actgrafdat, &joborgdat.grafdat, pooldat.mappptr, domnsubtab, vflowgttab) != 0) { /* Create bipartition graph */
+      if (kgraphMapRbBgraph (dataptr, &actgrafdat, &joborgdat.grafdat, pooldat.mappptr, domnsubtab, vflowgttab, contptr) != 0) { /* Create bipartition graph */
         errorPrint ("kgraphMapRbMap: cannot create bipartition graph");
         kgraphMapRbMapPoolExit (&pooldat);        /* Copied graph will be freed as not yet removed */
         return (1);
@@ -956,7 +962,7 @@ KgraphMapRbVflo * restrict const        vflotab)  /*+ Array of fixed vertex load
         errorPrint             ("kgraphMapRbMap: cannot bipartition job");
         bgraphExit             (&actgrafdat);
         kgraphMapRbMapPoolExit (&pooldat);
-        return                 (1);
+        return (1);
       }
 
       if ((partval = 1, actgrafdat.compsize0 == 0) || /* If no bipartition found */
@@ -964,32 +970,29 @@ KgraphMapRbVflo * restrict const        vflotab)  /*+ Array of fixed vertex load
         if ((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) != 0) { /* If architecture is variable-sized       */
           pooldat.domntab[0][jobsubnum[0]] = joborgdat.domnorg; /* Propagate domain in next pool         */
           kgraphMapRbMapPoolRemv (&pooldat, &joborgdat); /* Remove job from pool as long as graph exists */
-          bgraphExit (&actgrafdat);               /* Free bipartitioning data as well as current graph   */
-          continue;                               /* Process next job in current pool                    */
         }
-        if (archDomSize (mappptr->archptr, &domnsubtab[partval]) <= 1) { /* If domain is terminal        */
+        else if (archDomSize (mappptr->archptr, &domnsubtab[partval]) <= 1) { /* If domain is terminal   */
           pooldat.domntab[0][jobsubnum[0]] = domnsubtab[partval]; /* Refine domain in next pool          */
           kgraphMapRbMapPoolRemv (&pooldat, &joborgdat); /* Remove job from pool as long as graph exists */
-          bgraphExit (&actgrafdat);               /* Free bipartitioning data as well as current graph   */
-          continue;                               /* Process next job in current pool                    */
         }
         else {                                    /* Re-use job slot and graph for further bipartitioning */
           pooldat.domntab[0][jobsubnum[0]] =      /* Update domain in next pool                           */
           joborgptr->domnorg = domnsubtab[partval]; /* New job takes same graph and non-empty subdomain   */
           joborgptr->vflonbr = vflonbrtab[partval];
-          joborgptr->vflotab = joborgdat.vflotab + (partval * vflonbrtab[0]); /* Point to proper sub-array           */
+          joborgptr->vflotab = joborgdat.vflotab + (partval * vflonbrtab[0]); /* Point to proper sub-array */
+          joborgptr->levlnum ++;
           kgraphMapRbMapPoolUpdt1 (&pooldat, &joborgdat, actgrafdat.parttax, joborgptr, partval); /* Add job to pool */
           actgrafdat.s.flagval &= ~GRAPHFREETABS; /* Since graph will be re-used, never free its internal arrays     */
-          bgraphExit (&actgrafdat);               /* Free bipartitioning data                                        */
-          continue;                               /* Process next job in current pool                                */
         }
+        bgraphExit (&actgrafdat);                 /* Free bipartitioning data as well as current graph */
+        continue;                                 /* Process next job in current pool                  */
       }
 
       if ((pooldat.mappptr->domnnbr == pooldat.mappptr->domnmax) && /* If all job slots busy and if cannot resize */
-          (kgraphMapRbMapPoolResize (&pooldat) != 0)) {
+          (kgraphMapRbMapPoolResize (&pooldat) != 0)) { /* From this point, joborgptr may no longer be valid      */
         errorPrint             ("kgraphMapRbMap: cannot resize structures");
         kgraphMapRbMapPoolExit (&pooldat);
-        return                 (1);
+        return (1);
       }
 
       jobsubnum[1] = pooldat.mappptr->domnnbr ++; /* Get slot number of new subdomain */
@@ -1006,6 +1009,7 @@ KgraphMapRbVflo * restrict const        vflotab)  /*+ Array of fixed vertex load
       else
         kgraphMapRbMapPartBoth (&pooldat, &actgrafdat, jobsubnum);
 
+      levlnum = joborgdat.levlnum + 1;
       for (i = 1; i >= 0; i --) {                 /* For both subdomains */
         KgraphMapRbMapJob * jobsubptr;
 
@@ -1027,12 +1031,13 @@ KgraphMapRbVflo * restrict const        vflotab)  /*+ Array of fixed vertex load
           errorPrint             ("kgraphMapRbMap: cannot create induced subgraph");
           bgraphExit             (&actgrafdat);
           kgraphMapRbMapPoolExit (&pooldat);
-          return                 (1);
+          return (1);
         }
         jobsubptr->poolflag = 1;                  /* So that graph is freed in case of error on other part */
         jobsubptr->domnorg  = domnsubtab[i];
         jobsubptr->vflonbr  = vflonbrtab[i];
         jobsubptr->vflotab  = joborgdat.vflotab + (i * vflonbrtab[0]); /* Point to proper sub-array */
+        jobsubptr->levlnum  = levlnum;            /* Set new level */
       }
 
       if ((jobsubsiz[0] | jobsubsiz[1]) == 0)     /* If both subjobs do not need further processing */
