@@ -1,4 +1,4 @@
-/* Copyright 2008,2011 ENSEIRB, INRIA & CNRS
+/* Copyright 2008,2011,2014,2018,2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -41,9 +41,11 @@
 /**                mapping algorithm.                      **/
 /**                                                        **/
 /**   DATES      : # Version 5.1  : from : 22 sep 2008     **/
-/**                                 to     14 apr 2011     **/
+/**                                 to   : 14 apr 2011     **/
 /**                # Version 6.0  : from : 03 mar 2011     **/
-/**                                 to     25 oct 2011     **/
+/**                                 to   : 07 jun 2018     **/
+/**                # Version 7.0  : from : 06 may 2021     **/
+/**                                 to   : 27 jul 2021     **/
 /**                                                        **/
 /************************************************************/
 
@@ -51,34 +53,30 @@
 **  The type and structure definitions.
 */
 
-/*+ This structure holds the data passed to each bipartitioning job. +*/
+/*+ This structure holds the splitting parameters. +*/
 
-typedef struct KgraphMapRbPartData_ {
-  const Graph *             topgrafptr;           /*+ Pointer to top-level graph                     +*/
-  const Graph *             fixgrafptr;           /*+ Pointer to top-level graph with fixed vertices +*/
-  Gnum *                    rvnutax;              /*+ Ancestor to current vertex number array        +*/
-  Gnum *                    topfrontab;           /*+ Pointer to top-level frontier array            +*/
-  Gnum                      topfronnbr;           /*+ Current number of frontier vertices            +*/
-  Mapping *                 mappptr;              /*+ Current mapping of graph vertices              +*/
-  struct {                                        /*+ Remapping structure                            +*/
-    Mapping *               mappptr;              /*+ Old mapping (for remapping only)               +*/  
-    Gnum                    cmloval;              /*+ Migration edge load                            +*/
-    const Gnum *            vmlotax;              /*+ Array of vertex multiplicity                   +*/
-  }                         r;
-  const Anum *              pfixtax;              /*+ Fixed partition array                          +*/
-  const KgraphMapRbParam *  paraptr;
-  double                    comploadmin;          /*+ Minimum vertex load per target load            +*/
-  double                    comploadmax;          /*+ Maximum vertex load per target load            +*/
-} KgraphMapRbPartData;
+typedef struct KgraphMapRbPartSplit2_ {
+  Gnum                        vertnbr;            /*+ Number of vertices in part or in graph         +*/
+  Anum                        vflonbr;            /*+ Number of fixed vertex load slots              +*/
+  KgraphMapRbVflo * restrict  vflotab;            /*+ Array of fixed vertex load slots               +*/
+  ArchDom *                   domnptr;            /*+ Original domain to bipartition (to avoid lock) +*/
+} KgraphMapRbPartSplit2;
+
+typedef struct KgraphMapRbPartSplit_ {
+  KgraphMapRbPartSplit2             splttab[2];   /*+ Array of induced subgraph data           +*/
+  const KgraphMapRbData * restrict  dataptr;      /*+ Global mapping data                      +*/
+  const Graph * restrict            grafptr;      /*+ Graph to induce and bipartition          +*/
+  const GraphPart * restrict        parttax;      /*+ Part array of original graph to consider +*/
+  Gnum                              levlnum;      /*+ Recursion level number                   +*/
+  int *                             revaptr;      /*+ Pointer to return value                  +*/
+} KgraphMapRbPartSplit;
 
 /*
 **  The function prototypes.
 */
 
-#ifndef KGRAPH_MAP_RB_PART
-#define static
-#endif
+#ifdef KGRAPH_MAP_RB_PART
+static void                 kgraphMapRbPart2    (Context * restrict const, const int, KgraphMapRbPartSplit * const);
+#endif /* KGRAPH_MAP_RB_PART */
 
-int                         kgraphMapRbPart     (Kgraph * const, const KgraphMapRbParam * const, const Graph * const, Gnum * const);
-
-#undef static
+int                         kgraphMapRbPart     (const KgraphMapRbData * restrict const, const Graph * restrict const, const Anum, KgraphMapRbVflo * restrict, Context * restrict const);
