@@ -1,4 +1,4 @@
-/* Copyright 2007,2012,2018 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2007,2012,2018,2019 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -40,9 +40,11 @@
 /**                the libSCOTCH library.                  **/
 /**                                                        **/
 /**   DATES      : # Version 5.0  : from : 21 jul 2007     **/
-/**                                 to     04 aug 2007     **/
+/**                                 to   : 04 aug 2007     **/
 /**                # Version 6.0  : from : 29 nov 2012     **/
-/**                                 to     25 apr 2018     **/
+/**                                 to   : 25 apr 2018     **/
+/**                # Version 7.0  : from : 20 sep 2019     **/
+/**                                 to   : 20 sep 2019     **/
 /**                                                        **/
 /************************************************************/
 
@@ -54,6 +56,7 @@
 
 #include "module.h"
 #include "common.h"
+#include "context.h"
 #include "dgraph.h"
 #include "order.h"
 #include "dorder.h"
@@ -96,13 +99,13 @@ SCOTCH_Num * const          treetab)              /*+ Separator tree array
   }
 #endif /* SCOTCH_DEBUG_LIBRARY1 */
 
-  srcgrafptr = (Dgraph *) grafptr;                /* Use structure as distributed source graph */
+  srcgrafptr = (Dgraph *) CONTEXTOBJECT (grafptr); /* Use structure as distributed source graph */
   libcordptr = (LibOrder *) cordptr;
-  libcordptr->permtab = ((permtab == NULL) || ((void *) permtab == (void *) grafptr)) ? NULL : (Gnum *) permtab;
-  libcordptr->peritab = ((peritab == NULL) || ((void *) peritab == (void *) grafptr)) ? NULL : (Gnum *) peritab;
-  libcordptr->cblkptr = ((cblkptr == NULL) || ((void *) cblkptr == (void *) grafptr)) ? NULL : (Gnum *) cblkptr;
-  libcordptr->rangtab = ((rangtab == NULL) || ((void *) rangtab == (void *) grafptr)) ? NULL : (Gnum *) rangtab;
-  libcordptr->treetab = ((treetab == NULL) || ((void *) treetab == (void *) grafptr)) ? NULL : (Gnum *) treetab;
+  libcordptr->permtab = ((permtab == NULL) || ((void *) permtab == (void *) srcgrafptr)) ? NULL : (Gnum *) permtab;
+  libcordptr->peritab = ((peritab == NULL) || ((void *) peritab == (void *) srcgrafptr)) ? NULL : (Gnum *) peritab;
+  libcordptr->cblkptr = ((cblkptr == NULL) || ((void *) cblkptr == (void *) srcgrafptr)) ? NULL : (Gnum *) cblkptr;
+  libcordptr->rangtab = ((rangtab == NULL) || ((void *) rangtab == (void *) srcgrafptr)) ? NULL : (Gnum *) rangtab;
+  libcordptr->treetab = ((treetab == NULL) || ((void *) treetab == (void *) srcgrafptr)) ? NULL : (Gnum *) treetab;
 
   return (orderInit (&libcordptr->o, srcgrafptr->baseval, srcgrafptr->vertglbnbr, libcordptr->peritab));
 }
