@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -41,9 +41,11 @@
 /**                library.                                **/
 /**                                                        **/
 /**   DATES      : # Version 5.1  : from : 28 may 2008     **/
-/**                                 to     28 may 2008     **/
+/**                                 to   : 28 may 2008     **/
 /**                # Version 6.0  : from : 29 nov 2012     **/
-/**                                 to     29 nov 2012     **/
+/**                                 to   : 29 nov 2012     **/
+/**                # Version 7.0  : from : 20 sep 2019     **/
+/**                                 to   : 20 sep 2019     **/
 /**                                                        **/
 /************************************************************/
 
@@ -55,6 +57,7 @@
 
 #include "module.h"
 #include "common.h"
+#include "context.h"
 #include "dgraph.h"
 #include "dorder.h"
 #include "ptscotch.h"
@@ -81,5 +84,5 @@ const SCOTCH_Dgraph * const     grafptr,          /*+ Graph to order   +*/
 const SCOTCH_Dordering * const  ordeptr,          /*+ Ordering to save +*/
 FILE * const                    stream)           /*+ Output stream    +*/
 {
-  return (dorderSaveBlock ((Dorder *) ordeptr, (Dgraph *) grafptr, stream));
+  return (dorderSaveBlock ((Dorder *) ordeptr, (Dgraph *) CONTEXTOBJECT (grafptr), stream));
 }
