@@ -1,4 +1,4 @@
-/* Copyright 2007,2012 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2007,2012,2019 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -43,6 +43,8 @@
 /**                                 to   : 18 oct 2007     **/
 /**                # Version 6.0  : from : 29 nov 2012     **/
 /**                                 to   : 29 nov 2012     **/
+/**                # Version 7.0  : from : 20 sep 2019     **/
+/**                                 to   : 20 sep 2019     **/
 /**                                                        **/
 /************************************************************/
 
@@ -54,6 +56,7 @@
 
 #include "module.h"
 #include "common.h"
+#include "context.h"
 #include "dgraph.h"
 #include "dorder.h"
 #include "ptscotch.h"
@@ -80,5 +83,5 @@ const SCOTCH_Dgraph * const     grafptr,          /*+ Graph to order     +*/
 const SCOTCH_Dordering * const  ordeptr,          /*+ Computed ordering  +*/
 SCOTCH_Num * const              permloctab)       /*+ Direct permutation +*/
 {
-  return (dorderPerm ((Dorder *) ordeptr, (Dgraph *) grafptr, permloctab));
+  return (dorderPerm ((Dorder *) ordeptr, (Dgraph *) CONTEXTOBJECT (grafptr), permloctab));
 }
