@@ -1,4 +1,4 @@
-/* Copyright 2010,2012 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2010,2012,2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -43,6 +43,8 @@
 /**                                 to   : 17 nov 2010     **/
 /**                # Version 6.0  : from : 29 nov 2012     **/
 /**                                 to   : 29 nov 2012     **/
+/**                # Version 6.1  : from : 15 mar 2021     **/
+/**                                 to   : 15 mar 2021     **/
 /**                                                        **/
 /************************************************************/
 
@@ -65,9 +67,9 @@
 
 /*+ This routine reserves a memory area
 *** of a size sufficient to store a
-*** graph mapping structure.
+*** SCOTCH_Dmapping structure.
 *** It returns:
-*** - !NULL  : if the initialization succeeded.
+*** - !NULL  : if the allocation succeeded.
 *** - NULL   : on error.
 +*/
 
@@ -76,3 +78,15 @@ SCOTCH_dmapAlloc ()
 {
   return ((SCOTCH_Dmapping *) memAlloc (sizeof (SCOTCH_Dmapping)));
 }
+
+/*+ This routine returns the size, in bytes,
+*** of a SCOTCH_Dmapping structure.
+*** It returns:
+*** - > 0  : in all cases.
++*/
+
+int
+SCOTCH_dmapSizeof ()
+{
+  return (sizeof (SCOTCH_Dmapping));
+}
