@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010,2018,2019 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010,2018,2019,2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -51,6 +51,10 @@
 /**                                 to   : 17 nov 2010     **/
 /**                # Version 6.0  : from : 04 dec 2012     **/
 /**                                 to   : 24 jul 2019     **/
+/**                # Version 6.1  : from : 15 mar 2021     **/
+/**                                 to   : 31 may 2021     **/
+/**                # Version 7.0  : from : 07 may 2019     **/
+/**                                 to   : 21 aug 2019     **/
 /**                                                        **/
 /************************************************************/
 
@@ -62,6 +66,7 @@
 
 #include "module.h"
 #include "common.h"
+#include "context.h"
 #include "graph.h"
 #include "graph_io.h"
 #include "scotch.h"
@@ -75,9 +80,9 @@
 
 /*+ This routine reserves a memory area
 *** of a size sufficient to store a
-*** centralized graph structure.
+*** SCOTCH_Graph structure.
 *** It returns:
-*** - !NULL  : if the initialization succeeded.
+*** - !NULL  : if the allocation succeeded.
 *** - NULL   : on error.
 +*/
 
@@ -87,6 +92,18 @@ SCOTCH_graphAlloc ()
   return ((SCOTCH_Graph *) memAlloc (sizeof (SCOTCH_Graph)));
 }
 
+/*+ This routine returns the size, in bytes,
+*** of a SCOTCH_Graph structure.
+*** It returns:
+*** - > 0  : in all cases.
++*/
+
+int
+SCOTCH_graphSizeof ()
+{
+  return (sizeof (SCOTCH_Graph));
+}
+
 /*+ This routine initializes the opaque
 *** graph structure used to handle graphs
 *** in the Scotch library.
@@ -121,7 +138,8 @@ void
 SCOTCH_graphExit (
 SCOTCH_Graph * const        grafptr)
 {
-  graphExit ((Graph *) grafptr);
+  if (! contextContainerTrue (grafptr))
+    graphExit ((Graph *) grafptr);
 }
 
 /*+ This routine frees the contents of the
@@ -134,7 +152,8 @@ void
 SCOTCH_graphFree (
 SCOTCH_Graph * const        grafptr)
 {
-  graphFree ((Graph *) grafptr);
+  if (! contextContainerTrue (grafptr))
+    graphFree ((Graph *) grafptr);
 }
 
 /*+ This routine loads the given opaque graph
@@ -159,19 +178,23 @@ const SCOTCH_Num            flagval)
 {
   GraphFlag           srcgrafflag;                /* Graph flags */
 
-  if ((baseval < -1) || (baseval > 1)) {
-    errorPrint (STRINGIFY (SCOTCH_graphLoad) ": invalid base parameter");
-    return     (1);
-  }
   if ((flagval < 0) || (flagval > 3)) {
     errorPrint (STRINGIFY (SCOTCH_graphLoad) ": invalid flag parameter");
-    return     (1);
+    return (1);
   }
+  if (baseval < -1) {
+    errorPrint (STRINGIFY (SCOTCH_graphLoad) ": invalid base parameter");
+    return (1);
+  }
+#ifdef SCOTCH_DEBUG_GRAPH1
+  if (baseval > 1)
+    errorPrintW (STRINGIFY (SCOTCH_graphLoad) ": unusual base parameter");
+#endif /* SCOTCH_DEBUG_GRAPH1 */
 
   srcgrafflag = (((flagval & 1) != 0) ? GRAPHIONOLOADVERT : 0) +
                 (((flagval & 2) != 0) ? GRAPHIONOLOADEDGE : 0);
 
-  return (graphLoad ((Graph * const) grafptr, stream, (Gnum) baseval, srcgrafflag));
+  return (graphLoad ((Graph * const) CONTEXTOBJECT (grafptr), stream, (Gnum) baseval, srcgrafflag));
 }
 
 /*+ This routine saves the contents of the given
@@ -186,7 +209,7 @@ SCOTCH_graphSave (
 const SCOTCH_Graph * const  grafptr,
 FILE * const                stream)
 {
-  return (graphSave ((const Graph * const) grafptr, stream));
+  return (graphSave ((const Graph * const) CONTEXTOBJECT (grafptr), stream));
 }
 
 /*+ This routine fills the contents of the given
@@ -220,13 +243,11 @@ const SCOTCH_Num * const    edlotab)              /* Edge load array
     errorPrint (STRINGIFY (SCOTCH_graphBuild) ": internal error");
     return     (1);
   }
+  if ((baseval < 0) || (baseval > 1))
+    errorPrintW (STRINGIFY (SCOTCH_graphBuild) ": non-standard base parameter");
 #endif /* SCOTCH_DEBUG_LIBRARY1 */
-  if ((baseval < 0) || (baseval > 1)) {
-    errorPrint (STRINGIFY (SCOTCH_graphBuild) ": invalid base parameter");
-    return     (1);
-  }
 
-  srcgrafptr = (Graph *) grafptr;                 /* Use structure as source graph */
+  srcgrafptr = (Graph *) CONTEXTOBJECT (grafptr); /* Use structure as source graph */
   srcgrafptr->flagval = GRAPHNONE;
   srcgrafptr->baseval = baseval;
   srcgrafptr->vertnbr = vertnbr;
@@ -298,7 +319,7 @@ SCOTCH_Num * const          edgenbr)
 {
   const Graph *       srcgrafptr;
 
-  srcgrafptr = (Graph *) grafptr;
+  srcgrafptr = (Graph *) CONTEXTOBJECT (grafptr);
 
   if (vertnbr != NULL)
     *vertnbr = (SCOTCH_Num) (srcgrafptr->vertnbr);
@@ -329,7 +350,7 @@ SCOTCH_Num ** const         edlotab)              /* Edge load array          */
 {
   const Graph *       srcgrafptr;                 /* Pointer to source graph structure */
 
-  srcgrafptr = (const Graph *) grafptr;
+  srcgrafptr = (const Graph *) CONTEXTOBJECT (grafptr);
 
   if (baseptr != NULL)
     *baseptr = srcgrafptr->baseval;
@@ -394,7 +415,7 @@ double *                    edlodltptr)
   double              edloavg;
   double              edlodlt;
 
-  srcgrafptr = (Graph *) grafptr;
+  srcgrafptr = (Graph *) CONTEXTOBJECT (grafptr);
 
   vertnbr = srcgrafptr->vertnnd - srcgrafptr->baseval;
 
