@@ -1,4 +1,4 @@
-/* Copyright 2017,2018 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2017-2019 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -40,7 +40,9 @@
 /**                libSCOTCH library.                      **/
 /**                                                        **/
 /**   DATES      : # Version 6.0  : from : 26 jan 2017     **/
-/**                                 to     22 apr 2018     **/
+/**                                 to   : 22 apr 2018     **/
+/**                # Version 7.0  : from : 07 may 2019     **/
+/**                                 to   : 12 sep 2019     **/
 /**                                                        **/
 /************************************************************/
 
@@ -52,6 +54,7 @@
 
 #include "module.h"
 #include "common.h"
+#include "context.h"
 #include "graph.h"
 #include "scotch.h"
 
@@ -72,7 +75,18 @@
 
 SCOTCH_Num
 SCOTCH_graphDiamPV (
-const SCOTCH_Graph * const  grafptr)
+const SCOTCH_Graph * const  libgrafptr)
 {
-  return ((SCOTCH_Num) graphDiamPV ((Graph * const) grafptr));
+  CONTEXTDECL        (libgrafptr);
+  SCOTCH_Num          diamval;
+
+  if (CONTEXTINIT (libgrafptr) != 0) {
+    errorPrint (STRINGIFY (SCOTCH_graphDiamPV) ": cannot initialize context");
+    return     (1);
+  }
+
+  diamval = (SCOTCH_Num) graphDiamPV ((Graph * const) CONTEXTGETOBJECT (libgrafptr), CONTEXTGETDATA (libgrafptr));
+
+  CONTEXTEXIT (libgrafptr);
+  return      (diamval);
 }
