@@ -1,4 +1,4 @@
-/* Copyright 2008-2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2008-2012,2014,2018-2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -44,7 +44,9 @@
 /**                # Version 5.1  : from : 26 oct 2008     **/
 /**                                 to   : 31 aug 2011     **/
 /**                # Version 6.0  : from : 01 jan 2012     **/
-/**                                 to   : 10 jul 2018     **/
+/**                                 to   : 17 apr 2019     **/
+/**                # Version 7.0  : from : 03 sep 2020     **/
+/**                                 to   : 24 oct 2021     **/
 /**                                                        **/
 /************************************************************/
 
@@ -81,6 +83,12 @@ static const char *         C_usageList[] = {     /* Usage */
   "dgmap [<input source file> [<input target file> [<output mapping file> [<output log file>]]]] <options>",
   "dgpart [<nparts/pwght>] [<input source file> [<output mapping file> [<output log file>]]] <options>",
   "  -b<val>    : Load imbalance tolerance (default: 0.05)",
+  "  -C<opt>    : Choose execution context options according to one or several of <opt>:",
+  "                 d  : deterministic behavior (even across multiple threads; implies 'f')",
+  "                 f  : fixed random seed",
+  "                 r  : variable random seed",
+  "                 u  : undeterministic behavior (may be faster with several threads)",
+  "                 Default behavior depends on compilation flags",
   "  -c<opt>    : Choose default mapping strategy according to one or several of <opt>:",
   "                 b  : enforce load balance as much as possible",
   "                 q  : privilege quality over speed (default)",
@@ -116,6 +124,8 @@ main (
 int                         argc,
 char *                      argv[])
 {
+  SCOTCH_Context      contdat;                    /* Execution context         */
+  SCOTCH_Dgraph       cogrdat;                    /* Context graph binding     */
   SCOTCH_Dgraph       grafdat;                    /* Source graph              */
   SCOTCH_Num          grafflag;                   /* Source graph properties   */
   SCOTCH_Arch         archdat;                    /* Target architecture       */
@@ -134,10 +144,10 @@ char *                      argv[])
   MPI_Datatype        redutype;
   MPI_Op              reduop;
   int                 i, j;
-#ifdef SCOTCH_PTHREAD
+#ifdef SCOTCH_PTHREAD_MPI
   int                 thrdlvlreqval;
   int                 thrdlvlproval;
-#endif /* SCOTCH_PTHREAD */
+#endif /* SCOTCH_PTHREAD_MPI */
 
   flagval = C_FLAGNONE;                           /* Default behavior  */
   kbalval = 0.05;                                 /* Default imbalance */
@@ -153,16 +163,16 @@ char *                      argv[])
   errorProg ("dgmap");
 #endif /* SCOTCH_COMPILE_PART */
 
-#ifdef SCOTCH_PTHREAD
+#ifdef SCOTCH_PTHREAD_MPI
   thrdlvlreqval = MPI_THREAD_MULTIPLE;
   if (MPI_Init_thread (&argc, &argv, thrdlvlreqval, &thrdlvlproval) != MPI_SUCCESS)
     errorPrint ("main: Cannot initialize (1)");
   if (thrdlvlreqval > thrdlvlproval)
-    errorPrint ("main: MPI implementation is not thread-safe: recompile without SCOTCH_PTHREAD");
-#else /* SCOTCH_PTHREAD */
+    errorPrint ("main: MPI implementation is not thread-safe: recompile without SCOTCH_PTHREAD_MPI");
+#else /* SCOTCH_PTHREAD_MPI */
   if (MPI_Init (&argc, &argv) != MPI_SUCCESS)
     errorPrint ("main: Cannot initialize (2)");
-#endif /* SCOTCH_PTHREAD */
+#endif /* SCOTCH_PTHREAD_MPI */
 
   MPI_Comm_size (MPI_COMM_WORLD, &procglbnbr);    /* Get communicator data */
   MPI_Comm_rank (MPI_COMM_WORLD, &proclocnum);
@@ -170,13 +180,14 @@ char *                      argv[])
 
   if ((argc >= 2) && (argv[1][0] == '?')) {       /* If need for help */
     usagePrint (stdout, C_usageList);
-    return     (0);
+    return     (EXIT_SUCCESS);
   }
 
   SCOTCH_randomProc (proclocnum);                 /* Record process number to initialize pseudo-random seed */
 
   grafflag = 0;                                   /* Use vertex and edge weights */
-  SCOTCH_stratInit (&stradat);
+  SCOTCH_contextInit (&contdat);                  /* Set default context          */
+  SCOTCH_stratInit   (&stradat);
 
   fileBlockInit (C_fileTab, C_FILENBR);           /* Set default stream pointers */
 
@@ -207,6 +218,9 @@ char *                      argv[])
           }
           break;
         case 'C' :
+          if (SCOTCH_contextOptionParse (&contdat, &argv[i][2]) != 0)
+            errorPrint ("main: invalid context option string");
+          break;
         case 'c' :                                /* Strategy selection parameters */
           for (j = 2; argv[i][j] != '\0'; j ++) {
             switch (argv[i][j]) {
@@ -244,7 +258,7 @@ char *                      argv[])
         case 'H' :                                /* Give the usage message */
         case 'h' :
           usagePrint (stdout, C_usageList);
-          return     (0);
+          return     (EXIT_SUCCESS);
         case 'M' :
         case 'm' :
           straptr = &argv[i][2];
@@ -293,9 +307,9 @@ char *                      argv[])
           break;
         case 'V' :
           fprintf (stderr, "dgmap/dgpart, version " SCOTCH_VERSION_STRING "\n");
-          fprintf (stderr, "Copyright 2008-2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS, France\n");
-          fprintf (stderr, "This software is libre/free software under CeCILL-C -- see the user's manual for more information\n");
-          return  (0);
+          fprintf (stderr, SCOTCH_COPYRIGHT_STRING "\n");
+          fprintf (stderr, SCOTCH_LICENSE_STRING "\n");
+          return  (EXIT_SUCCESS);
         case 'v' :                                /* Output control info */
           for (j = 2; argv[i][j] != '\0'; j ++) {
             switch (argv[i][j]) {
@@ -383,6 +397,8 @@ char *                      argv[])
   else
     SCOTCH_stratDgraphMapBuild (&stradat, straval, (SCOTCH_Num) procglbnbr, (SCOTCH_Num) C_partNbr, kbalval);
 
+  SCOTCH_contextBindDgraph (&contdat, &grafdat, &cogrdat);
+
   clockStop (&runtime[0]);                        /* Get input time */
   clockInit (&runtime[1]);
 
@@ -396,7 +412,7 @@ char *                      argv[])
   SCOTCH_dgraphGhst (&grafdat);                   /* Compute it once for good */
 
   SCOTCH_dgraphMapInit    (&grafdat, &mappdat, &archdat, NULL);
-  SCOTCH_dgraphMapCompute (&grafdat, &mappdat, &stradat); /* Perform mapping */
+  SCOTCH_dgraphMapCompute (&cogrdat, &mappdat, &stradat); /* Perform mapping */
 
   clockStop (&runtime[1]);                        /* Get computation time */
 
@@ -468,13 +484,15 @@ char *                      argv[])
   fileBlockClose (C_fileTab, C_FILENBR);          /* Always close explicitely to end eventual (un)compression tasks */
 
   SCOTCH_dgraphMapExit (&grafdat, &mappdat);
+  SCOTCH_dgraphExit    (&cogrdat);                /* Destroy context binding first */
   SCOTCH_dgraphExit    (&grafdat);
   SCOTCH_stratExit     (&stradat);
   SCOTCH_archExit      (&archdat);
+  SCOTCH_contextExit   (&contdat);
 
   MPI_Finalize ();
 
-  return (0);
+  return (EXIT_SUCCESS);
 }
 
 /* Reduction routine for statistics output.
