@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010-2012,2014,2018,2019 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010-2012,2014,2018,2019,2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -39,9 +39,9 @@
 /**                graphs.                                 **/
 /**                                                        **/
 /**   DATES      : # Version 2.0  : from : 30 oct 1994     **/
-/**                                 to     08 nov 1994     **/
+/**                                 to   : 08 nov 1994     **/
 /**                # Version 3.0  : from : 11 jul 1995     **/
-/**                                 to     02 oct 1995     **/
+/**                                 to   : 02 oct 1995     **/
 /**                # Version 3.2  : from : 03 jun 1997     **/
 /**                                 to   : 03 jun 1997     **/
 /**                # Version 3.3  : from : 06 oct 1998     **/
@@ -53,7 +53,11 @@
 /**                # Version 5.1  : from : 01 jul 2010     **/
 /**                                 to   : 14 feb 2011     **/
 /**                # Version 6.0  : from : 01 jan 2012     **/
-/**                                 to   : 17 apr 2019     **/
+/**                                 to   : 24 sep 2019     **/
+/**                # Version 6.1  : from : 02 apr 2021     **/
+/**                                 to   : 02 apr 2021     **/
+/**                # Version 7.0  : from : 02 apr 2021     **/
+/**                                 to   : 02 apr 2021     **/
 /**                                                        **/
 /**   NOTES      : # The vertices of the (dX,dY) mesh are  **/
 /**                  numbered as terminals so that         **/
@@ -61,6 +65,13 @@
 /**                  t(dX - 1, 0) = dX - 1, t(0,1) = dX,   **/
 /**                  and t(x,y) = (y * dX) + x.            **/
 /**                                                        **/
+/**                # The bit mask array represents the     **/
+/**                  possible neighbors of a vertex, in    **/
+/**                  the following order:                  **/
+/**                         1     2     4                  **/
+/**                         8   (16)   32                  **/
+/**                        64   128   256                  **/
+/**                                                        **/
 /************************************************************/
 
 /*
@@ -90,8 +101,9 @@ static const char *         C_usageList[] = {
   "  -e        : Build a 8-neighbor grid rather than a 4-neighbor one",
   "  -g<file>  : Output the geometry to <file>",
   "  -h        : Display this help",
-  "  -t        : Build a torus rather than a mesh",
+  "  -t        : Build a torus rather than a grid",
   "  -V        : Print program version and copyright",
+  "  -y        : Invert y coordinate in geometry",
   NULL };
 
 /****************************************/
@@ -106,16 +118,19 @@ main (
 int                         argc,
 char *                      argv[])
 {
-  int                 flagval;                    /* Process flags      */
-  SCOTCH_Num          baseval;                    /* Base value         */
-  SCOTCH_Num          d[2] = { 1, 1 };            /* Mesh dimensions    */
-  SCOTCH_Num          c[2];                       /* Vertex coordinates */
+  int                 flagval;                    /* Process flags          */
+  SCOTCH_Num          baseval;                    /* Base value             */
+  SCOTCH_Num          edgenbr;                    /* Number of edges (arcs) */
+  SCOTCH_Num          d[2] = { 1, 1 };            /* Mesh dimensions        */
+  SCOTCH_Num          c[2];                       /* Vertex coordinates     */
+  int                 cbitmsk;                    /* Coordinate flag mask   */
   int                 i;
 
   errorProg ("gmk_m2");
 
   flagval = C_FLAGDEFAULT;                        /* Set default flags */
   baseval = 0;
+  cbitmsk = 2 + 8 + 32 + 128;                     /* 4-point mask */
 
   if ((argc >= 2) && (argv[1][0] == '?')) {       /* If need for help */
     usagePrint (stdout, C_usageList);
@@ -127,18 +142,14 @@ char *                      argv[])
   for (i = 1; i < argc; i ++) {                   /* Loop for all option codes                        */
     if ((argv[i][0] != '-') || (argv[i][1] == '\0') || (argv[i][1] == '.')) { /* If found a file name */
       if (C_paraNum < 2) {                        /* If number of parameters not reached              */
-        if ((d[C_paraNum ++] = atoi (argv[i])) < 1) { /* Get the dimension                            */
+        if ((d[C_paraNum ++] = atoi (argv[i])) < 1) /* Get the dimension                              */
           errorPrint ("main: invalid dimension '%s'", argv[i]);
-          return     (EXIT_FAILURE);
-        }
         continue;                                 /* Process the other parameters */
       }
       if (C_fileNum < C_FILEARGNBR)               /* A file name has been given */
         fileBlockName (C_fileTab, C_fileNum ++) = argv[i];
-      else {
+      else
         errorPrint ("main: too many file names given");
-        return     (EXIT_FAILURE);
-      }
     }
     else {                                        /* If found an option name */
       switch (argv[i][1]) {
@@ -152,6 +163,7 @@ char *                      argv[])
         case 'E' :                                /* Build a finite-element grid */
         case 'e' :
           flagval |= C_FLAGELEM;
+          cbitmsk  = 1 + 2 + 4 + 8 + 32 + 64 + 128 + 256; /* 8-point mask */
           break;
         case 'G' :                                /* Output the geometry */
         case 'g' :
@@ -172,75 +184,73 @@ char *                      argv[])
           fprintf (stderr, SCOTCH_COPYRIGHT_STRING "\n");
           fprintf (stderr, SCOTCH_LICENSE_STRING "\n");
           return  (EXIT_SUCCESS);
+        case 'Y' :                                /* Invert the y coordinate in geometry file */
+        case 'y' :
+          flagval |= C_FLAGGEOINVY;
+          break;
         default :
           errorPrint ("main: unprocessed option '%s'", argv[i]);
-          return     (EXIT_FAILURE);
       }
     }
   }
 
   fileBlockOpen (C_fileTab, C_FILENBR);           /* Open all files */
 
-  if (flagval & C_FLAGELEM) {                     /* Build a 8-neighbor grid */
-    errorPrint ("main: elements not supported");
-    return     (EXIT_FAILURE);
-  }
+  if ((flagval & C_FLAGTORUS) != 0) {             /* Build a torus           */
+    edgenbr = ((flagval & C_FLAGELEM) != 0)       /* Compute number of edges */
+              ? (8 * (d[0] * d[1]) - ((d[0] < 3) ? (6 * d[1]) : 0) - ((d[1] < 3) ? (6 * d[0]) : 0))
+              : (4 * (d[0] * d[1]) - ((d[0] < 3) ? (2 * d[1]) : 0) - ((d[1] < 3) ? (2 * d[0]) : 0));
 
-  if (flagval & C_FLAGTORUS) {                    /* Build a torus */
     fprintf (C_filepntrsrcout, "0\n" SCOTCH_NUMSTRING "\t" SCOTCH_NUMSTRING "\n" SCOTCH_NUMSTRING "\t000\n",
-             (SCOTCH_Num) (d[0] * d[1]),          /* Print number of vertices              */
-             (SCOTCH_Num) ((4 * d[0] * d[1])             - /* Print number of edges (arcs) */
-                           ((d[0] < 3) ? (2 * d[1]) : 0) -
-                           ((d[1] < 3) ? (2 * d[0]) : 0)),
+             (SCOTCH_Num) (d[0] * d[1]),          /* Print number of vertices */
+             (SCOTCH_Num) edgenbr,
              (SCOTCH_Num) baseval);
 
     for (c[1] = 0; c[1] < d[1]; c[1] ++) {        /* Output neighbor list */
       for (c[0] = 0; c[0] < d[0]; c[0] ++) {
-        fprintf (C_filepntrsrcout, SCOTCH_NUMSTRING,
-                 (SCOTCH_Num) (((d[0] > 2) ? 3 : d[0]) + /* Output number of neighbors */
-                               ((d[1] > 2) ? 3 : d[1]) - 2));
-        if (d[1] > 2)
-          fprintf (C_filepntrsrcout, "\t" SCOTCH_NUMSTRING, /* Output the neighbors */
-                   (SCOTCH_Num) (((c[1] + d[1] - 1) % d[1]) * d[0] + c[0] + baseval));
-        if (d[0] > 2)
-          fprintf (C_filepntrsrcout, "\t" SCOTCH_NUMSTRING,
-                   (SCOTCH_Num) ((c[1] * d[0] + (c[0] + d[0] - 1) % d[0]) + baseval));
-        if (d[0] > 1)
-          fprintf (C_filepntrsrcout, "\t" SCOTCH_NUMSTRING,
-                   (SCOTCH_Num) (c[1] * d[0] + ((c[0] + 1) % d[0]) + baseval));
-        if (d[1] > 1)
-          fprintf (C_filepntrsrcout, "\t" SCOTCH_NUMSTRING,
-                   (SCOTCH_Num) (((c[1] + 1) % d[1]) * d[0] + c[0] + baseval));
-        fprintf (C_filepntrsrcout, "\n");
+        int                 cbitval;
+
+        cbitval = cbitmsk;
+	if (d[0] <= 2) {                          /* Remove ascending arcs if duplicates */
+          cbitval &= ~(4 + 32 + 256);
+          if (d[0] <= 1)                          /* Remove descending arcs if no width */
+            cbitval &= ~(1 + 8 + 64);
+        }
+        if (d[1] <= 2) {
+          cbitval &= ~(64 + 128 + 256);
+          if (d[1] <= 1)
+            cbitval &= ~(1 + 2 + 4);
+        }
+
+        C_edgeOutput (d, c, baseval, cbitval, C_filepntrsrcout);
       }
     }
   }
-  else {                                          /* Build a mesh */
+  else {                                          /* Build a grid            */
+    edgenbr = ((flagval & C_FLAGELEM) != 0)       /* Compute number of edges */
+              ? (8 * (d[0] * d[1]) - 6 * (d[0] + d[1]) + 4)
+              : (4 * (d[0] * d[1]) - 2 * (d[0] + d[1]));
+
     fprintf (C_filepntrsrcout, "0\n" SCOTCH_NUMSTRING "\t" SCOTCH_NUMSTRING "\n" SCOTCH_NUMSTRING "\t000\n",
-             (SCOTCH_Num) (d[0] * d[1]),
-             (SCOTCH_Num) ((d[0] * d[1] * 2 - (d[0] + d[1])) * 2),
+             (SCOTCH_Num) (d[0] * d[1]),          /* Print number of vertices */
+             (SCOTCH_Num) edgenbr,
              (SCOTCH_Num) baseval);
 
     for (c[1] = 0; c[1] < d[1]; c[1] ++) {        /* Output neighbor list */
       for (c[0] = 0; c[0] < d[0]; c[0] ++) {
-        fprintf (C_filepntrsrcout, "%d",
-                 ((c[0] == 0)          ? 0 : 1) + /* Output number of neighbors */
-                 ((c[0] == (d[0] - 1)) ? 0 : 1) +
-                 ((c[1] == 0)          ? 0 : 1) +
-                 ((c[1] == (d[1] - 1)) ? 0 : 1));
-        if (c[1] != 0)                            /* Output the neighbors */
-          fprintf (C_filepntrsrcout, "\t" SCOTCH_NUMSTRING,
-                   (SCOTCH_Num) ((c[1] - 1) * d[0] + c[0] + baseval));
-        if (c[0] != 0)
-          fprintf (C_filepntrsrcout, "\t" SCOTCH_NUMSTRING,
-                   (SCOTCH_Num) (c[1] * d[0] + (c[0] - 1) + baseval));
-        if (c[0] != (d[0] - 1))
-          fprintf (C_filepntrsrcout, "\t" SCOTCH_NUMSTRING,
-                   (SCOTCH_Num) (c[1] * d[0] + (c[0] + 1) + baseval));
-        if (c[1] != (d[1] - 1))
-          fprintf (C_filepntrsrcout, "\t" SCOTCH_NUMSTRING,
-                   (SCOTCH_Num) ((c[1] + 1) * d[0] + c[0] + baseval));
-        fprintf (C_filepntrsrcout, "\n");
+        int                 cbitval;
+
+        cbitval = cbitmsk;
+        if (c[0] <= 0)
+          cbitval &= ~(1 + 8 + 64);
+        if (c[0] >= (d[0] - 1))
+          cbitval &= ~(4 + 32 + 256);
+        if (c[1] <= 0)
+          cbitval &= ~(1 + 2 + 4);
+        if (c[1] >= (d[1] - 1))
+          cbitval &= ~(64 + 128 + 256);
+
+        C_edgeOutput (d, c, baseval, cbitval, C_filepntrsrcout);
       }
     }
   }
@@ -254,7 +264,7 @@ char *                      argv[])
         fprintf (C_filepntrgeoout, SCOTCH_NUMSTRING "\t" SCOTCH_NUMSTRING "\t" SCOTCH_NUMSTRING "\n",
                  (SCOTCH_Num) (c[1] * d[0] + c[0] + baseval),
                  (SCOTCH_Num) c[0],
-                 (SCOTCH_Num) (d[1] - 1 - c[1]));
+                 (SCOTCH_Num) (((flagval & C_FLAGGEOINVY) != 0) ? (d[1] - 1 - c[1]) : c[1]));
     }
   }
 
@@ -262,3 +272,38 @@ char *                      argv[])
 
   return (EXIT_SUCCESS);
 }
+
+/* This routine writes the neighbors of a 2D grid vertex.
+** It returns:
+** - void  : in all cases.
+*/
+
+void
+C_edgeOutput (
+const SCOTCH_Num * const    d,
+const SCOTCH_Num * const    c,
+const SCOTCH_Num            baseval,
+const int                   cbitval,
+FILE * const                fileptr)
+{
+  SCOTCH_Num          n[2];                       /* Neighbor loop indices */
+  int                 degrval;
+  int                 cbitnum;
+
+  for (cbitnum = 256, degrval = 0; cbitnum != 0; cbitnum >>= 1) { /* Count effective number of neighbors */
+    if ((cbitval & cbitnum) != 0)
+      degrval ++;
+  }
+
+  fprintf (fileptr, "%d", degrval);               /* Write number of neighbors */
+
+  for (n[1] = c[1] - 1, cbitnum = 1; n[1] <= c[1] + 1; n[1] ++) { /* Output the effective neighbors */
+    for (n[0] = c[0] - 1; n[0] <= c[0] + 1; n[0] ++, cbitnum <<= 1) {
+      if ((cbitval & cbitnum) != 0)
+        fprintf (fileptr, "\t" SCOTCH_NUMSTRING,
+                 (SCOTCH_Num) (((n[1] + d[1]) % d[1]) * d[0] + ((n[0] + d[0]) % d[0]) + baseval));
+    }
+  }
+
+  fprintf (fileptr, "\n");
+}
