@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010-2012,2014,2018,2019 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010-2012,2014,2018,2019,2021 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -8,13 +8,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -25,7 +25,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-C license and that you accept its terms.
 */
@@ -52,7 +52,11 @@
 /**                # Version 5.1  : from : 01 jul 2010     **/
 /**                                 to   : 14 feb 2011     **/
 /**                # Version 6.0  : from : 01 jan 2012     **/
-/**                                 to   : 17 apr 2019     **/
+/**                                 to   : 24 sep 2019     **/
+/**                # Version 6.1  : from : 30 oct 2019     **/
+/**                                 to   : 30 oct 2019     **/
+/**                # Version 7.0  : from : 23 oct 2021     **/
+/**                                 to   : 23 oct 2021     **/
 /**                                                        **/
 /************************************************************/
 
@@ -81,6 +85,12 @@ static File                 C_fileTab[C_FILENBR] = { /* File array
 
 static const char *         C_usageList[] = {
   "gord [<input source file> [<output ordering file> [<output log file>]]] <options>",
+  "  -C<opt>    : Choose execution context options according to one or several of <opt>:",
+  "                 d  : deterministic behavior (even across multiple threads; implies 'f')",
+  "                 f  : fixed random seed",
+  "                 r  : variable random seed",
+  "                 u  : undeterministic behavior (may be faster with several threads)",
+  "                 Default behavior depends on compilation flags",
   "  -c<opt>    : Choose default ordering strategy according to one or several of <opt>:",
   "                 b  : enforce load balance as much as possible",
   "                 q  : privilege quality over speed (default)",
@@ -109,15 +119,17 @@ main (
 int                         argc,
 char *                      argv[])
 {
-  SCOTCH_Num          vertnbr;                    /* Number of vertices */
-  SCOTCH_Graph        grafdat;                    /* Source graph       */
-  SCOTCH_Ordering     ordedat;                    /* Graph ordering     */
-  SCOTCH_Num *        permtab;                    /* Permutation array  */
-  SCOTCH_Strat        stradat;                    /* Ordering strategy  */
+  SCOTCH_Context      contdat;                    /* Execution context     */
+  SCOTCH_Graph        cogrdat;                    /* Context graph binding */
+  SCOTCH_Num          vertnbr;                    /* Number of vertices    */
+  SCOTCH_Graph        grafdat;                    /* Source graph          */
+  SCOTCH_Ordering     ordedat;                    /* Graph ordering        */
+  SCOTCH_Num *        permtab;                    /* Permutation array     */
+  SCOTCH_Strat        stradat;                    /* Ordering strategy     */
   SCOTCH_Num          straval;
   char *              straptr;
   int                 flagval;
-  Clock               runtime[2];                 /* Timing variables   */
+  Clock               runtime[2];                 /* Timing variables      */
   int                 i, j;
 
   errorProg ("gord");
@@ -130,7 +142,8 @@ char *                      argv[])
   flagval = C_FLAGNONE;                           /* Default behavior  */
   straval = 0;                                    /* No strategy flags */
   straptr = NULL;
-  SCOTCH_stratInit (&stradat);
+  SCOTCH_contextInit (&contdat);                  /* Set default context */
+  SCOTCH_stratInit   (&stradat);
 
   fileBlockInit (C_fileTab, C_FILENBR);           /* Set default stream pointers */
 
@@ -144,6 +157,9 @@ char *                      argv[])
     else {                                        /* If found an option name */
       switch (argv[i][1]) {
         case 'C' :
+          if (SCOTCH_contextOptionParse (&contdat, &argv[i][2]) != 0)
+            errorPrint ("main: invalid context option string");
+          break;
         case 'c' :                                /* Strategy selection parameters */
           for (j = 2; argv[i][j] != '\0'; j ++) {
             switch (argv[i][j]) {
@@ -223,9 +239,9 @@ char *                      argv[])
   clockInit  (&runtime[0]);
   clockStart (&runtime[0]);
 
-  SCOTCH_graphInit (&grafdat);                    /* Create graph structure    */
-  SCOTCH_graphLoad (&grafdat, C_filepntrsrcinp, -1, 2); /* Read source graph   */
-  SCOTCH_graphSize (&grafdat, &vertnbr, NULL);    /* Get graph characteristics */
+  SCOTCH_graphInit (&grafdat);                    /* Create graph structure                      */
+  SCOTCH_graphLoad (&grafdat, C_filepntrsrcinp, -1, 2); /* Read source graph with vertex weights */
+  SCOTCH_graphSize (&grafdat, &vertnbr, NULL);    /* Get graph characteristics                   */
 
   if (straval != 0) {
     if (straptr != NULL)
@@ -234,22 +250,22 @@ char *                      argv[])
     SCOTCH_stratGraphOrderBuild (&stradat, straval, 0, 0.2);
   }
 
+  SCOTCH_contextBindGraph (&contdat, &grafdat, &cogrdat);
+
   clockStop  (&runtime[0]);                       /* Get input time */
   clockInit  (&runtime[1]);
   clockStart (&runtime[1]);
 
-  if ((permtab = (SCOTCH_Num *) memAlloc (vertnbr * sizeof (SCOTCH_Num))) == NULL) {
+  if ((permtab = (SCOTCH_Num *) memAlloc (vertnbr * sizeof (SCOTCH_Num))) == NULL)
     errorPrint ("main: out of memory");
-    return     (EXIT_FAILURE);
-  }
+
   SCOTCH_graphOrderInit    (&grafdat, &ordedat, permtab, NULL, NULL, NULL, NULL); /* Create ordering */
-  SCOTCH_graphOrderCompute (&grafdat, &ordedat, &stradat); /* Perform ordering */
+  SCOTCH_graphOrderCompute (&cogrdat, &ordedat, &stradat); /* Perform ordering */
 
   clockStop (&runtime[1]);                        /* Get ordering time */
 
 #ifdef SCOTCH_DEBUG_ALL
-  if (SCOTCH_graphOrderCheck (&grafdat, &ordedat) != 0)
-    return (EXIT_FAILURE);
+  SCOTCH_graphOrderCheck (&grafdat, &ordedat);
 #endif /* SCOTCH_DEBUG_ALL */
 
   clockStart (&runtime[0]);
@@ -277,9 +293,11 @@ char *                      argv[])
 
   fileBlockClose (C_fileTab, C_FILENBR);          /* Always close explicitely to end eventual (un)compression tasks */
 
+  SCOTCH_graphExit      (&cogrdat);               /* Destroy context binding first */
   SCOTCH_graphOrderExit (&grafdat, &ordedat);
   SCOTCH_stratExit      (&stradat);
   SCOTCH_graphExit      (&grafdat);
+  SCOTCH_contextExit    (&contdat);
   memFree               (permtab);
 
   return (EXIT_SUCCESS);
