@@ -59,7 +59,9 @@
 /**                # Version 6.0  : from : 16 oct 2010     **/
 /**                                 to   : 17 apr 2019     **/
 /**                # Version 6.1  : from : 04 apr 2021     **/
-/**                                 to   : 04 apr 2021     **/
+/**                                 to   : 28 aug 2021     **/
+/**                # Version 7.0  : from : 31 aug 2021     **/
+/**                                 to   : 31 aug 2021     **/
 /**                                                        **/
 /************************************************************/
 
@@ -126,9 +128,12 @@ static const char *         C_usageList[] = {     /* Usage */
   "                          d        : display disks",
   "                          r        : remove cut edges",
   "                          v        : view cut edges",
+  "  -Ov[{<arguments>}]  : VTK mesh file :",
+  "                          r        : remove cut edges",
+  "                          v        : view cut edges",
   "  -V                  : Print program version and copyright",
   "",
-  "Default option set is : -Oi{c,v}",
+  "Default option set is : -Ov{v}",
   NULL };
 
 /*****************************/
